@@ -620,58 +620,19 @@ class ChromiumBase(BasePage):
         js = f'localStorage.getItem("{item}");' if item else 'localStorage;'
         return self.run_js_loaded(js, as_expr=True)
 
-    def get_screenshot(self, path=None, as_bytes=None, full_page=False, left_top=None, right_bottom=None):
+    def get_screenshot(self, path=None, as_bytes=None, as_base64=None,
+                       full_page=False, left_top=None, right_bottom=None):
         """对页面进行截图，可对整个网页、可见网页、指定范围截图。对可视范围外截图需要90以上版本浏览器支持
         :param path: 完整路径，后缀可选 'jpg','jpeg','png','webp'
-        :param as_bytes: 是否已字节形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数无效
+        :param as_bytes: 是否以字节形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数和as_base64参数无效
+        :param as_base64: 是否以base64字符串形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数无效
         :param full_page: 是否整页截图，为True截取整个网页，为False截取可视窗口
         :param left_top: 截取范围左上角坐标
         :param right_bottom: 截取范围右下角角坐标
         :return: 图片完整路径或字节文本
         """
-        if as_bytes:
-            if as_bytes is True:
-                pic_type = 'png'
-            else:
-                if as_bytes not in ('jpg', 'jpeg', 'png', 'webp'):
-                    raise ValueError("只能接收 'jpg', 'jpeg', 'png', 'webp' 四种格式。")
-                pic_type = 'jpeg' if as_bytes == 'jpg' else as_bytes
-
-        else:
-            if not path:
-                path = f'{self.title}.jpg'
-            path = get_usable_path(path)
-            pic_type = path.suffix.lower()
-            if pic_type not in ('.jpg', '.jpeg', '.png', '.webp'):
-                raise TypeError(f'不支持的文件格式：{pic_type}。')
-            pic_type = 'jpeg' if pic_type == '.jpg' else pic_type[1:]
-
-        width, height = self.size
-        if full_page:
-            vp = {'x': 0, 'y': 0, 'width': width, 'height': height, 'scale': 1}
-            png = self.run_cdp_loaded('Page.captureScreenshot', format=pic_type,
-                                      captureBeyondViewport=True, clip=vp)['data']
-        else:
-            if left_top and right_bottom:
-                x, y = left_top
-                w = right_bottom[0] - x
-                h = right_bottom[1] - y
-                vp = {'x': x, 'y': y, 'width': w, 'height': h, 'scale': 1}
-                png = self.run_cdp_loaded('Page.captureScreenshot', format=pic_type,
-                                          captureBeyondViewport=False, clip=vp)['data']
-            else:
-                png = self.run_cdp_loaded('Page.captureScreenshot', format=pic_type)['data']
-
-        from base64 import b64decode
-        png = b64decode(png)
-
-        if as_bytes:
-            return png
-
-        path.parent.mkdir(parents=True, exist_ok=True)
-        with open(path, 'wb') as f:
-            f.write(png)
-        return str(path.absolute())
+        return self._get_screenshot(path=path, as_bytes=as_bytes, as_base64=as_base64,
+                                    full_page=full_page, left_top=left_top, right_bottom=right_bottom)
 
     def clear_cache(self, session_storage=True, local_storage=True, cache=True, cookies=True):
         """清除缓存，可选要清除的项
@@ -736,6 +697,73 @@ class ChromiumBase(BasePage):
 
         return True
 
+    def _get_screenshot(self, path=None, as_bytes=None, as_base64=None,
+                        full_page=False, left_top=None, right_bottom=None, ele=None):
+        """对页面进行截图，可对整个网页、可见网页、指定范围截图。对可视范围外截图需要90以上版本浏览器支持
+        :param path: 完整路径，后缀可选 'jpg','jpeg','png','webp'
+        :param as_bytes: 是否以字节形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数和as_base64参数无效
+        :param as_base64: 是否以base64字符串形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数无效
+        :param full_page: 是否整页截图，为True截取整个网页，为False截取可视窗口
+        :param left_top: 截取范围左上角坐标
+        :param right_bottom: 截取范围右下角角坐标
+        :param ele: 为异域iframe内元素截图设置
+        :return: 图片完整路径或字节文本
+        """
+        if as_bytes:
+            if as_bytes is True:
+                pic_type = 'png'
+            else:
+                if as_bytes not in ('jpg', 'jpeg', 'png', 'webp'):
+                    raise ValueError("只能接收 'jpg', 'jpeg', 'png', 'webp' 四种格式。")
+                pic_type = 'jpeg' if as_bytes == 'jpg' else as_bytes
+
+        elif as_base64:
+            if as_base64 is True:
+                pic_type = 'png'
+            else:
+                if as_base64 not in ('jpg', 'jpeg', 'png', 'webp'):
+                    raise ValueError("只能接收 'jpg', 'jpeg', 'png', 'webp' 四种格式。")
+                pic_type = 'jpeg' if as_base64 == 'jpg' else as_base64
+
+        else:
+            if not path:
+                path = f'{self.title}.jpg'
+            path = get_usable_path(path)
+            pic_type = path.suffix.lower()
+            if pic_type not in ('.jpg', '.jpeg', '.png', '.webp'):
+                raise TypeError(f'不支持的文件格式：{pic_type}。')
+            pic_type = 'jpeg' if pic_type == '.jpg' else pic_type[1:]
+
+        width, height = self.size
+        if full_page:
+            vp = {'x': 0, 'y': 0, 'width': width, 'height': height, 'scale': 1}
+            png = self.run_cdp_loaded('Page.captureScreenshot', format=pic_type,
+                                      captureBeyondViewport=True, clip=vp)['data']
+        else:
+            if left_top and right_bottom:
+                x, y = left_top
+                w = right_bottom[0] - x
+                h = right_bottom[1] - y
+                vp = {'x': x, 'y': y, 'width': w, 'height': h, 'scale': 1}
+                png = self.run_cdp_loaded('Page.captureScreenshot', format=pic_type,
+                                          captureBeyondViewport=False, clip=vp)['data']
+            else:
+                png = self.run_cdp_loaded('Page.captureScreenshot', format=pic_type)['data']
+
+        if as_base64:
+            return png
+
+        from base64 import b64decode
+        png = b64decode(png)
+
+        if as_bytes:
+            return png
+
+        path.parent.mkdir(parents=True, exist_ok=True)
+        with open(path, 'wb') as f:
+            f.write(png)
+        return str(path.absolute())
+
     # ------------------准备废弃----------------------
     def wait_loading(self, timeout=None):
         """阻塞程序，等待页面进入加载状态
@@ -903,6 +931,11 @@ class ChromiumBaseSetter(object):
         """返回用于设置页面加载策略的对象"""
         return PageLoadStrategy(self._page)
 
+    @property
+    def scroll(self):
+        """返回用于设置页面滚动设置的对象"""
+        return PageScrollSetter(self._page.scroll)
+
     def timeouts(self, implicit=None, page_load=None, script=None):
         """设置超时时间，单位为秒
         :param implicit: 查找元素超时时间
@@ -1061,16 +1094,30 @@ class ChromiumPageScroll(ChromiumScroll):
         self.t1 = 'window'
         self.t2 = 'document.documentElement'
 
-    def to_see(self, loc_or_ele):
+    def to_see(self, loc_or_ele, center=False):
         """滚动页面直到元素可见
         :param loc_or_ele: 元素的定位信息，可以是loc元组，或查询字符串
+        :param center: 是否尽量滚动到页面正中
         :return: None
         """
         ele = self._driver._ele(loc_or_ele)
-        ele.run_js('this.scrollIntoView({behavior: "auto", block: "nearest", inline: "nearest"});')
+        self._to_see(ele, center)
 
+    def _to_see(self, ele, center):
+        """执行滚动页面直到元素可见
+        :param ele: 元素对象
+        :param center: 是否尽量滚动到页面正中
+        :return: None
+        """
+        if center:
+            ele.run_js('this.scrollIntoViewIfNeeded();')
+            self._wait_scrolled()
+            return
+
+        ele.run_js('this.scrollIntoViewIfNeeded(false);')
         if ele.states.is_covered:
-            ele.run_js('this.scrollIntoView({behavior: "auto", block: "center", inline: "center"});')
+            ele.run_js('this.scrollIntoViewIfNeeded();')
+        self._wait_scrolled()
 
 
 class Timeout(object):
@@ -1121,3 +1168,27 @@ class PageLoadStrategy(object):
     def none(self):
         """设置页面加载策略为none"""
         self._page._page_load_strategy = 'none'
+
+
+class PageScrollSetter(object):
+    def __init__(self, scroll):
+        self._scroll = scroll
+
+    def wait_complete(self, on_off=True):
+        """设置滚动命令后是否等待完成
+        :param on_off: 开或关
+        :return: None
+        """
+        if not isinstance(on_off, bool):
+            raise TypeError('on_off必须为bool。')
+        self._scroll._wait_complete = on_off
+
+    def smooth(self, on_off=True):
+        """设置页面滚动是否平滑滚动
+        :param on_off: 开或关
+        :return: None
+        """
+        if not isinstance(on_off, bool):
+            raise TypeError('on_off必须为bool。')
+        b = 'smooth' if on_off else 'auto'
+        self._scroll._driver.run_js(f'document.documentElement.style.setProperty("scroll-behavior","{b}");')
