@@ -119,7 +119,7 @@ class ChromiumElement(DrissionElement):
     def size(self):
         """返回元素宽和高组成的元组"""
         model = self.page.run_cdp('DOM.getBoxModel', backendNodeId=self._backend_id)['model']
-        return model['height'], model['width']
+        return model['width'], model['height']
 
     @property
     def set(self):
@@ -172,7 +172,7 @@ class ChromiumElement(DrissionElement):
     def scroll(self):
         """用于滚动滚动条的对象"""
         if self._scroll is None:
-            self._scroll = ChromiumScroll(self)
+            self._scroll = ChromiumElementScroll(self)
         return self._scroll
 
     @property
@@ -429,6 +429,7 @@ class ChromiumElement(DrissionElement):
         while perf_counter() < end_time:
             try:
                 result = self.page.run_cdp('Page.getResourceContent', frameId=frame, url=src)
+                break
             except CallMethodError:
                 pass
             sleep(.1)
@@ -462,10 +463,11 @@ class ChromiumElement(DrissionElement):
         with open(f'{path}{sep}{rename}', write_type) as f:
             f.write(data)
 
-    def get_screenshot(self, path=None, as_bytes=None):
+    def get_screenshot(self, path=None, as_bytes=None, as_base64=None):
         """对当前元素截图，可保存到文件，或以字节方式返回
         :param path: 完整路径，后缀可选 'jpg','jpeg','png','webp'
-        :param as_bytes: 是否已字节形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数无效
+        :param as_bytes: 是否以字节形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数和as_base64参数无效
+        :param as_base64: 是否以base64字符串形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数无效
         :return: 图片完整路径或字节文本
         """
         if self.tag == 'img':  # 等待图片加载完成
@@ -476,16 +478,16 @@ class ChromiumElement(DrissionElement):
             while not self.run_js(js) and perf_counter() < end_time:
                 sleep(.1)
 
-        self.page.scroll.to_see(self)
+        self.scroll.to_see(center=True)
         sleep(1)
         left, top = self.location
-        height, width = self.size
+        width, height = self.size
         left_top = (left, top)
         right_bottom = (left + width, top + height)
         if not path:
             path = f'{self.tag}.jpg'
-        return self.page.get_screenshot(path, as_bytes=as_bytes, full_page=False,
-                                        left_top=left_top, right_bottom=right_bottom)
+        return self.page._get_screenshot(path, as_bytes=as_bytes, as_base64=as_base64, full_page=False,
+                                         left_top=left_top, right_bottom=right_bottom, ele=self)
 
     def input(self, vals, clear=True):
         """输入文本或组合键，也可用于输入文件路径到input元素（路径间用\n间隔）
@@ -1647,7 +1649,7 @@ class Click(object):
         """
         if not by_js:
             try:
-                self._ele.page.scroll.to_see(self._ele)
+                self._ele.scroll.to_see()
                 if self._ele.states.is_in_viewport and not self._ele.states.is_covered:
                     client_x, client_y = self._ele.locations.viewport_click_point
                     self._click(client_x, client_y)
@@ -1710,10 +1712,12 @@ class ChromiumScroll(object):
         """
         self._driver = ele
         self.t1 = self.t2 = 'this'
+        self._wait_complete = False
 
     def _run_js(self, js):
         js = js.format(self.t1, self.t2, self.t2)
         self._driver.run_js(js)
+        self._wait_scrolled()
 
     def to_top(self):
         """滚动到顶端，水平位置不变"""
@@ -1773,6 +1777,36 @@ class ChromiumScroll(object):
         """
         self._run_js(f'{{}}.scrollBy({pixel}, 0);')
 
+    def _wait_scrolled(self):
+        if not self._wait_complete:
+            return
+
+        page = self._driver.page if isinstance(self._driver, ChromiumElement) else self._driver
+        r = page.run_cdp('Page.getLayoutMetrics')
+        x = r['layoutViewport']['pageX']
+        y = r['layoutViewport']['pageY']
+
+        while True:
+            sleep(.1)
+            r = page.run_cdp('Page.getLayoutMetrics')
+            x1 = r['layoutViewport']['pageX']
+            y1 = r['layoutViewport']['pageY']
+
+            if x == x1 and y == y1:
+                break
+
+            x = x1
+            y = y1
+
+
+class ChromiumElementScroll(ChromiumScroll):
+    def to_see(self, center=False):
+        """滚动页面直到元素可见
+        :param center: 是否尽量滚动到页面正中
+        :return: None
+        """
+        self._driver.page.scroll.to_see(self._driver, center=center)
+
 
 class ChromiumSelect(object):
     """ChromiumSelect 类专门用于处理 d 模式下 select 标签"""
