@@ -7,6 +7,7 @@ from re import search
 from time import sleep, perf_counter
 from warnings import warn
 
+from .commons.tools import get_usable_path
 from .chromium_base import ChromiumBase, ChromiumPageScroll, ChromiumBaseSetter
 from .chromium_element import ChromiumElement
 
@@ -402,21 +403,80 @@ class ChromiumFrame(ChromiumBase):
         self._check_ok()
         return self.frame_ele.afters(filter_loc, timeout)
 
-    def get_screenshot(self, path=None, as_bytes=None, full_page=False, left_top=None, right_bottom=None):
+    def get_screenshot(self, path=None, as_bytes=None, as_base64=None):
         """对页面进行截图，可对整个网页、可见网页、指定范围截图。对可视范围外截图需要90以上版本浏览器支持
         :param path: 完整路径，后缀可选 'jpg','jpeg','png','webp'
-        :param as_bytes: 是否已字节形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数无效
+        :param as_bytes: 是否以字节形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数和as_base64参数无效
+        :param as_base64: 是否以base64字符串形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数无效
+        :return: 图片完整路径或字节文本
+        """
+        return self.frame_ele.get_screenshot(path=path, as_bytes=as_bytes, as_base64=as_base64)
+
+    def _get_screenshot(self, path=None, as_bytes: [bool, str] = None, as_base64: [bool, str] = None,
+                        full_page=False, left_top=None, right_bottom=None, ele=None):
+        """实现对元素截图
+        :param path: 完整路径，后缀可选 'jpg','jpeg','png','webp'
+        :param as_bytes: 是否以字节形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数和as_base64参数无效
+        :param as_base64: 是否以base64字符串形式返回图片，可选 'jpg','jpeg','png','webp'，生效时path参数无效
         :param full_page: 是否整页截图，为True截取整个网页，为False截取可视窗口
         :param left_top: 截取范围左上角坐标
         :param right_bottom: 截取范围右下角角坐标
+        :param ele: 为异域iframe内元素截图设置
         :return: 图片完整路径或字节文本
         """
-        if full_page:
-            raise RuntimeError('暂未实现对iframe全页截图功能。')
-        if left_top is None and right_bottom is None:
-            return self.frame_ele.get_screenshot(path=path, as_bytes=as_bytes)
+        if not self._is_diff_domain:
+            return super().get_screenshot(path=path, as_bytes=as_bytes, as_base64=as_base64,
+                                          full_page=full_page, left_top=left_top, right_bottom=right_bottom)
+
+        if as_bytes:
+            if as_bytes is True:
+                pic_type = 'png'
+            else:
+                if as_bytes not in ('jpg', 'jpeg', 'png', 'webp'):
+                    raise ValueError("只能接收 'jpg', 'jpeg', 'png', 'webp' 四种格式。")
+                pic_type = 'jpeg' if as_bytes == 'jpg' else as_bytes
+
+        elif as_base64:
+            if as_base64 is True:
+                pic_type = 'png'
+            else:
+                if as_base64 not in ('jpg', 'jpeg', 'png', 'webp'):
+                    raise ValueError("只能接收 'jpg', 'jpeg', 'png', 'webp' 四种格式。")
+                pic_type = 'jpeg' if as_base64 == 'jpg' else as_base64
+
         else:
-            raise RuntimeError('暂未实现对异域iframe内元素截图功能。')
+            if not path:
+                path = f'{self.title}.jpg'
+            path = get_usable_path(path)
+            pic_type = path.suffix.lower()
+            if pic_type not in ('.jpg', '.jpeg', '.png', '.webp'):
+                raise TypeError(f'不支持的文件格式：{pic_type}。')
+            pic_type = 'jpeg' if pic_type == '.jpg' else pic_type[1:]
+
+        self.frame_ele.scroll.to_see(center=True)
+        self.scroll.to_see(ele, center=True)
+        cx, cy = ele.locations.viewport_location
+        w, h = ele.size
+        img_data = f'data:image/{pic_type};base64,{self.frame_ele.get_screenshot(as_base64=True)}'
+        body = self.page('t:body')
+        first_child = body('c::first-child')
+        if not isinstance(first_child, ChromiumElement):
+            first_child = first_child.frame_ele
+        js = f'''
+        img = document.createElement('img');
+        img.src = "{img_data}";
+        img.style.setProperty("z-index",9999999);
+        img.style.setProperty("position","fixed");
+        arguments[0].insertBefore(img, this);
+        return img;'''
+        new_ele = first_child.run_js(js, body)
+        new_ele.scroll.to_see(True)
+        top = int(self.frame_ele.style('border-top').split('px')[0])
+        left = int(self.frame_ele.style('border-left').split('px')[0])
+        r = self.page.get_screenshot(path=path, as_bytes=as_bytes, as_base64=as_base64,
+                                     left_top=(cx + left, cy + top), right_bottom=(cx + w + left, cy + h + top))
+        self.page.remove_ele(new_ele)
+        return r
 
     def _find_elements(self, loc_or_ele, timeout=None, single=True, relative=False, raise_err=None):
         """在frame内查找单个元素
@@ -568,14 +628,16 @@ class ChromiumFrameScroll(ChromiumPageScroll):
         """
         self._driver = frame.doc_ele
         self.t1 = self.t2 = 'this.documentElement'
+        self._wait_complete = False
 
-    def to_see(self, loc_or_ele):
+    def to_see(self, loc_or_ele, center=False):
         """滚动页面直到元素可见
         :param loc_or_ele: 元素的定位信息，可以是loc元组，或查询字符串
+        :param center: 是否尽量滚动到页面正中
         :return: None
         """
         ele = loc_or_ele if isinstance(loc_or_ele, ChromiumElement) else self._driver._ele(loc_or_ele)
-        ele.run_js('this.scrollIntoView({behavior: "auto", block: "center", inline: "center"});')
+        self._to_see(ele, center)
 
 
 class ChromiumFrameSetter(ChromiumBaseSetter):
