@@ -5,7 +5,10 @@
 """
 from pathlib import Path
 
-from DrissionPage.functions.web import cookies_to_tuple
+from requests import Session
+from requests.structures import CaseInsensitiveDict
+
+from DrissionPage.commons.web import cookies_to_tuple, set_session_cookies
 from .options_manage import OptionsManager
 
 
@@ -35,9 +38,10 @@ class SessionOptions(object):
 
         self._del_set = set()  # 记录要从ini文件删除的参数
 
-        if read_file:
-            self.ini_path = str(ini_path) if ini_path else str(Path(__file__).parent / 'configs.ini')
-            om = OptionsManager(self.ini_path)
+        if read_file is not False:
+            ini_path = str(ini_path) if ini_path else None
+            om = OptionsManager(ini_path)
+            self.ini_path = om.ini_path
             options_dict = om.session_options
 
             if options_dict.get('headers', None) is not None:
@@ -374,6 +378,27 @@ class SessionOptions(object):
         """以字典形式返回本对象"""
         return session_options_to_dict(self)
 
+    def make_session(self):
+        """根据内在的配置生成Session对象"""
+        s = Session()
+
+        if self.headers:
+            s.headers = CaseInsensitiveDict(self.headers)
+        if self.cookies:
+            set_session_cookies(s, self.cookies)
+        if self.adapters:
+            for url, adapter in self.adapters:
+                s.mount(url, adapter)
+
+        attrs = ['auth', 'proxies', 'hooks', 'params', 'verify',
+                 'cert', 'stream', 'trust_env', 'max_redirects']
+        for i in attrs:
+            attr = self.__getattribute__(i)
+            if attr:
+                s.__setattr__(i, attr)
+
+        return s
+
 
 def session_options_to_dict(options):
     """把session配置对象转换为字典
