@@ -10,8 +10,9 @@ from lxml.etree import tostring
 from lxml.html import HtmlElement, fromstring
 
 from .base import DrissionElement, BasePage, BaseElement
-from .functions.web import get_ele_txt, make_absolute_link
-from .functions.locator import get_loc
+from .commons.constants import NoneElement
+from .commons.locator import get_loc
+from .commons.web import get_ele_txt, make_absolute_link
 
 
 class SessionElement(DrissionElement):
@@ -212,12 +213,13 @@ class SessionElement(DrissionElement):
         """
         return self._ele(loc_or_str, single=False)
 
-    def _ele(self, loc_or_str, timeout=None, single=True, relative=False):
+    def _find_elements(self, loc_or_str, timeout=None, single=True, relative=False, raise_err=None):
         """返回当前元素下级符合条件的子元素、属性或节点文本，默认返回第一个
         :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
         :param timeout: 不起实际作用，用于和父类对应
         :param single: True则返回第一个，False则返回全部
         :param relative: WebPage用的表示是否相对定位的参数
+        :param raise_err: 找不到元素是是否抛出异常，为None时根据全局设置
         :return: SessionElement对象
         """
         return make_session_ele(self, loc_or_str, single)
@@ -317,7 +319,7 @@ def make_session_ele(html_or_ele, loc=None, single=True):
         page = None
         html_or_ele = fromstring(html_or_ele)
 
-    # ShadowRootElement, ChromiumShadowRootElement, ChromiumFrame
+    # ShadowRootElement, ChromiumShadowRoot, ChromiumFrame
     elif isinstance(html_or_ele, BaseElement) or the_type.endswith(".ChromiumFrame'>"):
         page = html_or_ele.page
         html_or_ele = fromstring(html_or_ele.html)
@@ -343,7 +345,7 @@ def make_session_ele(html_or_ele, loc=None, single=True):
             elif isinstance(ele, str):
                 return ele
             else:
-                return None
+                return NoneElement()
 
         else:  # 返回全部
             return [SessionElement(e, page) if isinstance(e, HtmlElement) else e for e in ele if e != '\n']
