@@ -4,7 +4,6 @@
 @Contact :   g1879@qq.com
 """
 from pathlib import Path
-from time import sleep
 from warnings import warn
 
 from requests import Session
@@ -12,13 +11,13 @@ from tldextract import extract
 
 from .base import BasePage
 from .chromium_base import ChromiumBase, Timeout
-from .chromium_driver import ChromiumDriver, CallMethodException
-from .chromium_page import ChromiumPage, ChromiumDownloadSetter
+from .chromium_driver import ChromiumDriver
+from .chromium_page import ChromiumPage, ChromiumDownloadSetter, ChromiumPageSetter
+from .chromium_tab import WebPageTab
 from .configs.chromium_options import ChromiumOptions
-from .configs.driver_options import DriverOptions
 from .configs.session_options import SessionOptions
-from .functions.web import cookies_to_tuple
-from .session_page import SessionPage
+from .errors import CallMethodError
+from .session_page import SessionPage, SessionPageSetter
 
 
 class WebPage(SessionPage, ChromiumPage, BasePage):
@@ -34,7 +33,8 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
         self._mode = mode.lower()
         if self._mode not in ('s', 'd'):
             raise ValueError('mode参数只能是s或d。')
-        self._has_driver, self._has_session = (None, True) if self._mode == 's' else (True, None)
+        self._has_driver = True
+        self._has_session = True
 
         self._debug = False
         self._debug_recorder = None
@@ -46,16 +46,18 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
         self._session_options = None
         self._response = None
         self._download_set = None
+        self._set = None
+        self._screencast = None
 
-        self._set_both_options(driver_or_options, session_or_options)
-
-        if self._mode == 'd':
-            self._to_d_mode()
+        self._set_start_options(driver_or_options, session_or_options)
+        self._set_runtime_settings()
+        self._connect_browser()
+        self._create_session()
 
         t = timeout if isinstance(timeout, (int, float)) else self.timeouts.implicit
         super(ChromiumBase, self).__init__(t)  # 调用Base的__init__()
 
-    def _set_both_options(self, dr_opt, se_opt):
+    def _set_start_options(self, dr_opt, se_opt):
         """处理两种模式的设置
         :param dr_opt: ChromiumDriver或DriverOptions对象，为None则从ini读取，为False用默认信息创建
         :param se_opt: Session、SessionOptions对象或配置信息，为None则从ini读取，为False用默认信息创建
@@ -63,9 +65,9 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
         """
         # 浏览器配置
         if isinstance(dr_opt, ChromiumDriver):
-            self._connect_browser(dr_opt)
-            self._has_driver = True
-            # self._driver_options = None
+            self._tab_obj = dr_opt
+            self._driver_options = ChromiumOptions()
+            self._driver_options.debugger_address = dr_opt.address
             dr_opt = False
 
         else:
@@ -75,17 +77,18 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
             elif dr_opt is False:
                 self._driver_options = ChromiumOptions(read_file=False)
 
-            elif isinstance(dr_opt, (ChromiumOptions, DriverOptions)):
+            elif str(type(dr_opt)).endswith(("ChromiumOptions'>", "DriverOptions'>")):
                 self._driver_options = dr_opt
 
             else:
                 raise TypeError('driver_or_options参数只能接收ChromiumDriver, ChromiumOptions、None或False。')
 
+        self.address = self._driver_options.debugger_address
+
         # Session配置
         if isinstance(se_opt, Session):
             self._session = se_opt
-            self._has_session = True
-            self._session_options = SessionOptions(read_file=False)
+            self._session_options = SessionOptions()
             se_opt = False
 
         else:
@@ -101,21 +104,21 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
             else:
                 raise TypeError('session_or_options参数只能接收Session, SessionOptions、None或False。')
 
-        # 通用配置
         self._timeouts = Timeout(self)
         self._page_load_strategy = self._driver_options.page_load_strategy
         self._download_path = None
+
         if se_opt is not False:
-            self.set_timeouts(implicit=self._session_options.timeout)
+            self.set.timeouts(implicit=self._session_options.timeout)
             self._download_path = self._session_options.download_path
 
         if dr_opt is not False:
             t = self._driver_options.timeouts
-            self.set_timeouts(t['implicit'], t['pageLoad'], t['script'])
+            self.set.timeouts(t['implicit'], t['pageLoad'], t['script'])
             self._download_path = self._driver_options.download_path
 
-    def _set_options(self):
-        """覆盖父类同名方法"""
+    def _set_runtime_settings(self):
+        """设置运行时用到的属性"""
         pass
 
     def __call__(self, loc_or_str, timeout=None):
@@ -138,6 +141,14 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
         elif self._mode == 's':
             return self._session_url
 
+    @property
+    def title(self):
+        """返回当前页面title"""
+        if self._mode == 's':
+            return super().title
+        elif self._mode == 'd':
+            return super(SessionPage, self).title
+
     @property
     def html(self):
         """返回页面html文本"""
@@ -157,7 +168,6 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
     @property
     def response(self):
         """返回 s 模式获取到的 Response 对象，切换到 s 模式"""
-        self.change_mode('s')
         return self._response
 
     @property
@@ -176,34 +186,9 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
     def session(self):
         """返回Session对象，如未初始化则按配置信息创建"""
         if self._session is None:
-            self._set_session(self._session_options)
-
+            self._create_session()
         return self._session
 
-    @property
-    def driver(self):
-        """返回纯粹的ChromiumDriver对象"""
-        return self._tab_obj
-
-    @property
-    def _wait_driver(self):
-        """返回用于控制浏览器的ChromiumDriver对象，会先等待页面加载完毕"""
-        while self._is_loading:
-            sleep(.1)
-        return self._driver
-
-    @property
-    def _driver(self):
-        """返回纯粹的ChromiumDriver对象，调用时切换到d模式，并连接浏览器"""
-        self.change_mode('d')
-        if self._tab_obj is None:
-            self._connect_browser(self._driver_options)
-        return self._tab_obj
-
-    @_driver.setter
-    def _driver(self, tab):
-        self._tab_obj = tab
-
     @property
     def _session_url(self):
         """返回 session 保存的url"""
@@ -220,7 +205,7 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
         :param second: 秒数
         :return: None
         """
-        self.set_timeouts(implicit=second)
+        self.set.timeouts(implicit=second)
 
     @property
     def download_path(self):
@@ -239,6 +224,13 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
         """返回下载器对象"""
         return self.download_set._switched_DownloadKit
 
+    @property
+    def set(self):
+        """返回用于等待的对象"""
+        if self._set is None:
+            self._set = WebPageSetter(self)
+        return self._set
+
     def get(self, url, show_errmsg=False, retry=None, interval=None, timeout=None, **kwargs):
         """跳转到一个url
         :param url: 目标url
@@ -266,7 +258,8 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
         :param kwargs: 连接参数
         :return: url是否可用
         """
-        self.change_mode('s', go=False)
+        if self.mode == 'd':
+            self.cookies_to_session()
         return super().post(url, data, show_errmsg, retry, interval, **kwargs)
 
     def ele(self, loc_or_ele, timeout=None):
@@ -335,7 +328,7 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
 
             if self._session_url:
                 if copy_cookies:
-                    self.cookies_to_driver()
+                    self.cookies_to_browser()
 
                 if go:
                     self.get(self._session_url)
@@ -360,13 +353,13 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
         :return: None
         """
         if copy_user_agent:
-            selenium_user_agent = self._tab_obj.Runtime.evaluate(expression='navigator.userAgent;')['result']['value']
+            selenium_user_agent = self.run_cdp('Runtime.evaluate', expression='navigator.userAgent;')['result']['value']
             self.session.headers.update({"User-Agent": selenium_user_agent})
 
-        self.set_cookies(self._get_driver_cookies(as_dict=True), set_session=True)
+        self.set.cookies(self._get_driver_cookies(as_dict=True), set_session=True)
 
-    def cookies_to_driver(self):
-        """把session对象的cookies复制到driver对象"""
+    def cookies_to_browser(self):
+        """把session对象的cookies复制到浏览器"""
         ex_url = extract(self._session_url)
         domain = f'{ex_url.domain}.{ex_url.suffix}'
         cookies = []
@@ -376,7 +369,7 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
 
             if domain in cookie['domain']:
                 cookies.append(cookie)
-        self.set_cookies(cookies, set_driver=True)
+        self.set.cookies(cookies, set_driver=True)
 
     def get_cookies(self, as_dict=False, all_domains=False):
         """返回cookies
@@ -389,56 +382,25 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
         elif self._mode == 'd':
             return self._get_driver_cookies(as_dict)
 
-    def set_user_agent(self, ua, platform=None):
-        """设置user agent，d模式下只有当前tab有效"""
-        super().set_user_agent(ua)
-        super(SessionPage, self).set_user_agent(ua, platform)
+    def get_tab(self, tab_id=None):
+        """获取一个标签页对象
+        :param tab_id: 要获取的标签页id，为None时获取当前tab
+        :return: 标签页对象
+        """
+        tab_id = tab_id or self.tab_id
+        return WebPageTab(self, tab_id)
 
     def _get_driver_cookies(self, as_dict=False):
         """获取浏览器cookies
         :param as_dict: 以dict形式返回
         :return: cookies信息
         """
-        cookies = self._tab_obj.Network.getCookies()['cookies']
+        cookies = self.run_cdp('Network.getCookies')['cookies']
         if as_dict:
             return {cookie['name']: cookie['value'] for cookie in cookies}
         else:
             return cookies
 
-    def set_cookies(self, cookies, set_session=False, set_driver=False):
-        """添加cookies信息到浏览器或session对象
-        :param cookies: 可以接收`CookieJar`、`list`、`tuple`、`str`、`dict`格式的`cookies`
-        :param set_session: 是否设置到Session对象
-        :param set_driver: 是否设置到浏览器
-        :return: None
-        """
-        # 添加cookie到driver
-        if set_driver:
-            cookies = cookies_to_tuple(cookies)
-            result_cookies = []
-            for cookie in cookies:
-                if not cookie.get('domain', None):
-                    continue
-                c = {'value': '' if cookie['value'] is None else cookie['value'],
-                     'name': cookie['name'],
-                     'domain': cookie['domain']}
-                result_cookies.append(c)
-            self._tab_obj.Network.setCookies(cookies=result_cookies)
-
-        # 添加cookie到session
-        if set_session:
-            super().set_cookies(cookies)
-
-    def set_headers(self, headers: dict) -> None:
-        """设置固定发送的headers
-        :param headers: dict格式的headers数据
-        :return: None
-        """
-        if self._has_session:
-            return super().set_headers(headers)
-        if self._has_driver:
-            super(SessionPage, self).set_headers(headers)
-
     def close_driver(self):
         """关闭driver及浏览器"""
         if self._has_driver:
@@ -447,6 +409,8 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
                 self.driver.Browser.close()
             except Exception:
                 pass
+            self._tab_obj.stop()
+            self._tab_obj = None
             self._has_driver = None
 
     def close_session(self):
@@ -458,17 +422,20 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
             self._response = None
             self._has_session = None
 
-    def _ele(self, loc_or_ele, timeout=None, single=True, relative=False):
+    def _find_elements(self, loc_or_ele, timeout=None, single=True, relative=False, raise_err=None):
         """返回页面中符合条件的元素、属性或节点文本，默认返回第一个
         :param loc_or_ele: 元素的定位信息，可以是元素对象，loc元组，或查询字符串
         :param timeout: 查找元素超时时间，d模式专用
         :param single: True则返回第一个，False则返回全部
+        :param relative: WebPage用的表示是否相对定位的参数
+        :param raise_err: 找不到元素是是否抛出异常，为None时根据全局设置
         :return: 元素对象或属性、文本节点文本
         """
         if self._mode == 's':
-            return super()._ele(loc_or_ele, single=single)
+            return super()._find_elements(loc_or_ele, single=single)
         elif self._mode == 'd':
-            return super(SessionPage, self)._ele(loc_or_ele, timeout=timeout, single=single, relative=relative)
+            return super(SessionPage, self)._find_elements(loc_or_ele, timeout=timeout, single=single,
+                                                           relative=relative)
 
     def quit(self):
         """关闭浏览器，关闭session"""
@@ -483,6 +450,67 @@ class WebPage(SessionPage, ChromiumPage, BasePage):
             self._tab_obj = None
             self._has_driver = None
 
+    # ---------------准备废弃---------------
+    def set_cookies(self, cookies, set_session=False, set_driver=False):
+        """添加cookies信息到浏览器或session对象
+        :param cookies: 可以接收`CookieJar`、`list`、`tuple`、`str`、`dict`格式的`cookies`
+        :param set_session: 是否设置到Session对象
+        :param set_driver: 是否设置到浏览器
+        :return: None
+        """
+        # 添加cookie到driver
+        warn("set_cookies()方法即将弃用，请用set.user_agent()方法代替。", DeprecationWarning)
+        self.set.cookies(cookies, set_session, set_driver)
+
+    def set_headers(self, headers) -> None:
+        """设置固定发送的headers
+        :param headers: dict格式的headers数据
+        :return: None
+        """
+        warn("set_headers()方法即将弃用，请用set.headers()方法代替。", DeprecationWarning)
+        self.set.headers(headers)
+
+    def set_user_agent(self, ua, platform=None):
+        """设置user agent，d模式下只有当前tab有效"""
+        warn("set_user_agent()方法即将弃用，请用set.user_agent()方法代替。", DeprecationWarning)
+        self.set.user_agent(ua, platform)
+
+
+class WebPageSetter(ChromiumPageSetter):
+    def __init__(self, page):
+        super().__init__(page)
+        self._session_setter = SessionPageSetter(self._page)
+        self._chromium_setter = ChromiumPageSetter(self._page)
+
+    def cookies(self, cookies, set_session=False, set_driver=False):
+        """添加cookies信息到浏览器或session对象
+        :param cookies: 可以接收`CookieJar`、`list`、`tuple`、`str`、`dict`格式的`cookies`
+        :param set_session: 是否设置到Session对象
+        :param set_driver: 是否设置到浏览器
+        :return: None
+        """
+        if set_driver and self._page._has_driver:
+            self._chromium_setter.cookies(cookies)
+        if set_session and self._page._has_session:
+            self._session_setter.cookies(cookies)
+
+    def headers(self, headers) -> None:
+        """设置固定发送的headers
+        :param headers: dict格式的headers数据
+        :return: None
+        """
+        if self._page._has_session:
+            self._session_setter.headers(headers)
+        if self._page._has_driver:
+            self._chromium_setter.headers(headers)
+
+    def user_agent(self, ua, platform=None):
+        """设置user agent，d模式下只有当前tab有效"""
+        if self._page._has_session:
+            self._session_setter.user_agent(ua)
+        if self._page._has_driver:
+            self._chromium_setter.user_agent(ua, platform)
+
 
 class WebPageDownloadSetter(ChromiumDownloadSetter):
     """用于设置下载参数的类"""
@@ -514,10 +542,9 @@ class WebPageDownloadSetter(ChromiumDownloadSetter):
             try:
                 self._page.browser_driver.Browser.setDownloadBehavior(behavior=self._behavior, downloadPath=path,
                                                                       eventsEnabled=True)
-            except CallMethodException:
+            except CallMethodError:
                 warn('\n您的浏览器版本太低，用新标签页下载文件可能崩溃，建议升级。')
-                self._page.run_cdp('Page.setDownloadBehavior', behavior=self._behavior, downloadPath=path,
-                                   not_change=True)
+                self._page.run_cdp('Page.setDownloadBehavior', behavior=self._behavior, downloadPath=path)
 
     def by_browser(self):
         """设置使用浏览器下载文件"""
@@ -529,7 +556,7 @@ class WebPageDownloadSetter(ChromiumDownloadSetter):
                                                                   downloadPath=self._page.download_path)
             self._page.browser_driver.Browser.downloadWillBegin = self._download_by_browser
 
-        except CallMethodException:
+        except CallMethodError:
             warn('\n您的浏览器版本太低，用新标签页下载文件可能崩溃，建议升级。')
             self._page.driver.Page.setDownloadBehavior(behavior='allow', downloadPath=self._page.download_path)
             self._page.driver.Page.downloadWillBegin = self._download_by_browser
@@ -542,7 +569,7 @@ class WebPageDownloadSetter(ChromiumDownloadSetter):
             try:
                 self._page.browser_driver.Browser.setDownloadBehavior(behavior='deny', eventsEnabled=True)
                 self._page.browser_driver.Browser.downloadWillBegin = self._download_by_DownloadKit
-            except CallMethodException:
+            except CallMethodError:
                 raise RuntimeError('您的浏览器版本太低，不支持此方法，请升级。')
 
         self._behavior = 'deny'
